@@ -5,18 +5,19 @@
  * of the gnome-shell source code
  */
 
-const AltTab = imports.ui.altTab;
 const Clutter = imports.gi.Clutter;
 const Gdk = imports.gi.Gdk;
 const Gio = imports.gi.Gio;
 const Gtk = imports.gi.Gtk;
-const GLib = imports.gi.GLib;
 const Lang = imports.lang;
-const Main = imports.ui.main;
 const Mainloop = imports.mainloop;
-const ModalDialog = imports.ui.modalDialog;
+const Meta = imports.gi.Meta;
 const Shell = imports.gi.Shell;
 const St = imports.gi.St;
+
+const AltTab = imports.ui.altTab;
+const Main = imports.ui.main;
+const ModalDialog = imports.ui.modalDialog;
 const Tweener = imports.ui.tweener;
 const WindowManager = imports.ui.windowManager;
 
@@ -24,55 +25,20 @@ const Gettext = imports.gettext.domain('gnome-shell-extensions');
 const _ = Gettext.gettext;
 const N_ = function(e) { return e };
 
+const ExtensionUtils = imports.misc.extensionUtils;
+const Me = ExtensionUtils.getCurrentExtension();
+const Convenience = Me.imports.convenience;
+
+let settings;
+
 const POPUP_DELAY_TIMEOUT = 150; // milliseconds
 
-// Settings: choose one of MODES, the description is in MESSAGE
-// (master branch has a nice dialog, but we cannot in gnome 3.2)
-const BEHAVIOUR = 'all_thumbnails';
+const SETTINGS_BEHAVIOUR_KEY = 'behaviour';
+const SETTINGS_HIGHLIGHT_SELECTED_KEY = 'highlight-selected';
 
-const MODES = {
-    all_thumbnails: function(shellwm, binding, mask, window, backwards) {
-        let tabPopup = new AltTabPopup2();
-
-        if (!tabPopup.show(backwards, binding, mask))
-            tabPopup.destroy();
-    },
-
-    workspace_icons: function(shellwm, binding, mask, window, backwards) {
-        if (shellwm._workspaceSwitcherPopup != null)
-            shellwm._workspaceSwitcherPopup.actor.hide();
-
-        let tabPopup = new AltTabPopupW();
-        if (!tabPopup.show(backwards, binding, mask))
-            tabPopup.destroy();
-    }
-};
-
-const MESSAGE = N_("This is the first time you use the Alternate Tab extension. \n\
-Please choose your preferred behaviour:\n\
-\n\
-All & Thumbnails:\n\
-    This mode presents all applications from all workspaces in one selection \n\
-    list. Instead of using the application icon of every window, it uses small \n\
-    thumbnails resembling the window itself. \n\
-\n\
-Workspace & Icons:\n\
-    This mode let's you switch between the applications of your current \n\
-    workspace and gives you additionally the option to switch to the last used \n\
-    application of your previous workspace. This is always the last symbol in \n\
-    the list and is segregated by a separator/vertical line if available. \n\
-    Every window is represented by its application icon.  \n\
-\n\
-If you whish to revert to the default behavior for the Alt-Tab switcher, just\n\
-disable the extension from extensions.gnome.org or the Advanced Settings application.\
-");
-
-function AltTabPopupW() {
-    this._init();
-}
-
-AltTabPopupW.prototype = {
-    __proto__ : AltTab.AltTabPopup.prototype,
+const AltTabPopupWorkspaceIcons = new Lang.Class({
+    Name: 'AlternateTab.AltTabPopupWorkspaceIcons',
+    Extends: AltTab.AltTabPopup,
 
     _windowActivated : function(thumbnailList, n) { },
 
@@ -111,6 +77,8 @@ AltTabPopupW.prototype = {
         this.actor.show();
         this.actor.get_allocation_box();
 
+        this._highlight_selected = settings.get_boolean(SETTINGS_HIGHLIGHT_SELECTED_KEY);
+
         // Make the initial selection
         if (binding == 'switch_group') {
             //see AltTab.AltTabPopup.show function
@@ -152,24 +120,69 @@ AltTabPopupW.prototype = {
         return true;
     },
 
+    _select : function(app, window, forceAppFocus) {
+        if (app != this._currentApp || window == null) {
+            if (this._thumbnails)
+                this._destroyThumbnails();
+        }
+
+        if (this._thumbnailTimeoutId != 0) {
+            Mainloop.source_remove(this._thumbnailTimeoutId);
+            this._thumbnailTimeoutId = 0;
+        }
+
+        this._thumbnailsFocused = (window != null) && !forceAppFocus;
+
+        this._currentApp = app;
+        this._currentWindow = window ? window : -1;
+        this._appSwitcher.highlight(app, this._thumbnailsFocused);
+
+        if (window != null) {
+            if (!this._thumbnails)
+                this._createThumbnails();
+            this._currentWindow = window;
+            this._thumbnails.highlight(window, forceAppFocus);
+        } else if (this._appIcons[this._currentApp].cachedWindows.length > 1 &&
+                   !forceAppFocus) {
+            this._thumbnailTimeoutId = Mainloop.timeout_add (
+                THUMBNAIL_POPUP_TIME,
+                Lang.bind(this, this._timeoutPopupThumbnails));
+        }
+        if (this._highlight_selected) {
+            let current_app = this._appIcons[this._currentApp];
+            Main.activateWindow(current_app.cachedWindows[0]);
+        }
+    },
 
     _finish : function() {
         let app = this._appIcons[this._currentApp];
         if (!app)
             return;
 
+        /*
+         * We've to restore the original Z-depth and order of all windows.
+         *
+         * Gnome-shell doesn't give an option to change Z-depth without
+         * messing the window's user_time.
+         *
+         * Pointless if the popup wasn't showed.
+         */
+        if (this._highlight_selected && this.actor.opacity == 255) {
+            for (let i = this._appIcons.length - 2; i >= 0; i--) {
+                let app_walker = this._appIcons[i];
+                Main.activateWindow(app_walker.cachedWindows[0], global.get_current_time() - i - 1);
+            }
+        }
+
         Main.activateWindow(app.cachedWindows[0]);
         this.destroy();
     }
 
-};
+});
 
-function AppIcon(app, window) {
-    this._init(app, window);
-}
-
-AppIcon.prototype = {
-    __proto__ : AltTab.AppIcon.prototype,
+const AppIcon = new Lang.Class({
+    Name: 'AlternateTab.AppIcon',
+    Extends: AltTab.AppIcon,
 
     _init: function(app, window) {
         this.app = app;
@@ -196,16 +209,16 @@ AppIcon.prototype = {
             this.actor.add(this.label, { x_fill: false });
         }
     }
-};
+});
 
-function WindowSwitcher(apps, altTabPopup) {
-    this._init(apps, altTabPopup);
-}
-
-WindowSwitcher.prototype = {
-    __proto__ : AltTab.AppSwitcher.prototype,
+const WindowSwitcher = new Lang.Class({
+    Name: 'AlternateTab.WindowSwitcher',
+    Extends: AltTab.AppSwitcher,
 
     _init : function(apps, altTabPopup) {
+        // Horrible HACK!
+        // We inherit from AltTab.AppSwitcher, but only chain up to
+        // AltTab.SwitcherList._init, to bypass AltTab.AppSwitcher._init
         AltTab.SwitcherList.prototype._init.call(this, true);
 
         // Construct the AppIcons, sort by time, add to the popup
@@ -265,84 +278,11 @@ WindowSwitcher.prototype = {
         if (t2 > t1) return 1;
         else return -1;
     }
-};
+});
 
-/* This object is never instantiated in the current branch, but
-   I don't trust git merge enough to remove it */
-function AltTabSettingsDialog() {
-    this._init();
-}
-
-AltTabSettingsDialog.prototype = {
-    __proto__: ModalDialog.ModalDialog.prototype,
-
-    _init : function() {
-        ModalDialog.ModalDialog.prototype._init.call(this, { styleClass: null });
-
-        let mainContentBox = new St.BoxLayout({ style_class: 'polkit-dialog-main-layout',
-                                                vertical: false });
-        this.contentLayout.add(mainContentBox,
-                               { x_fill: true,
-                                 y_fill: true });
-
-        let messageBox = new St.BoxLayout({ style_class: 'polkit-dialog-message-layout',
-                                            vertical: true });
-        mainContentBox.add(messageBox,
-                           { y_align: St.Align.START });
-
-        this._subjectLabel = new St.Label({ style_class: 'polkit-dialog-headline',
-                                            text: _("Alt Tab Behaviour") });
-
-        messageBox.add(this._subjectLabel,
-                       { y_fill:  false,
-                         y_align: St.Align.START });
-
-        this._descriptionLabel = new St.Label({ style_class: 'polkit-dialog-description',
-                                                text: Gettext.gettext(MESSAGE) });
-
-        messageBox.add(this._descriptionLabel,
-                       { y_fill:  true,
-                         y_align: St.Align.START });
-
-
-        this.setButtons([
-            {
-                label: _("All & Thumbnails"),
-                action: Lang.bind(this, function() {
-                    this.setBehaviour('all_thumbnails');
-                    this.close();
-                })
-            },
-            {
-                label: _("Workspace & Icons"),
-                action: Lang.bind(this, function() {
-                    this.setBehaviour('workspace_icons');
-                    this.close();
-                })
-            },
-            {
-                label: _("Cancel"),
-                action: Lang.bind(this, function() {
-                    this.close();
-                }),
-                key: Clutter.Escape
-            }
-        ]);
-    },
-
-    setBehaviour: function(behaviour) {
-           this._settings = new Gio.Settings({ schema: SETTINGS_SCHEMA });
-           this._settings.set_string(SETTINGS_BEHAVIOUR_KEY, behaviour);
-           this._settings.set_boolean(SETTINGS_FIRST_TIME_KEY, false);
-    }
-};
-
-function AltTabPopup2() {
-    this._init();
-}
-
-AltTabPopup2.prototype = {
-    __proto__ : AltTab.AltTabPopup.prototype,
+const AltTabPopupAllThumbnails = new Lang.Class({
+    Name: 'AlternateTab.AltTabPopup.AllThumbnails',
+    Extends: AltTab.AltTabPopup,
 
     _init : function() {
         this.actor = new Shell.GenericContainer({ name: 'altTabPopup',
@@ -465,52 +405,6 @@ AltTabPopup2.prototype = {
 	return true
     },
 
-    _keyPressEvent : function(actor, event) {
-        let keysym = event.get_key_symbol();
-        let shift = (Shell.get_event_state(event) & Clutter.ModifierType.SHIFT_MASK);
-        // X allows servers to represent Shift+Tab in two different ways
-        if (shift && keysym == Clutter.Tab)
-            keysym = Clutter.ISO_Left_Tab;
-
-        this._disableHover();
-
-        if (keysym == Clutter.grave)
-            this._select(this._currentApp, this._nextWindow());
-        else if (keysym == Clutter.asciitilde)
-            this._select(this._currentApp, this._previousWindow());
-        else if (keysym == Clutter.Escape)
-            this.destroy();
-        else if (this._thumbnailsFocused) {
-            if (keysym == Clutter.Tab) {
-                if (this._currentWindow == this._appIcons[this._currentApp].cachedWindows.length - 1)
-                    this._select(this._nextApp());
-                else
-                    this._select(this._currentApp, this._nextWindow());
-            } else if (keysym == Clutter.ISO_Left_Tab) {
-                if (this._currentWindow == 0 || this._currentWindow == -1)
-                    this._select(this._previousApp());
-                else
-                    this._select(this._currentApp, this._previousWindow());
-            } else if (keysym == Clutter.Left)
-                this._select(this._currentApp, this._previousWindow());
-            else if (keysym == Clutter.Right)
-                this._select(this._currentApp, this._nextWindow());
-            else if (keysym == Clutter.Up)
-                this._select(this._currentApp, null, true);
-        } else {
-            if (keysym == Clutter.Tab)
-                this._select(this._nextApp());
-            else if (keysym == Clutter.ISO_Left_Tab)
-                this._select(this._previousApp());
-            else if (keysym == Clutter.Left)
-                this._select(this._previousApp());
-            else if (keysym == Clutter.Right)
-                this._select(this._nextApp());
-        }
-
-        return true;
-    },
-
     _sortWindows : function(win1,win2) {
         let t1 = win1.get_user_time();
         let t2 = win2.get_user_time();
@@ -529,17 +423,14 @@ AltTabPopup2.prototype = {
         Main.activateWindow(app.cachedWindows[0]);
         this.destroy();
     },
-};
+});
 
-function WindowList(windows) {
-    this._init(windows);
-}
-
-WindowList.prototype = {
-    __proto__ : AltTab.SwitcherList.prototype,
+const WindowList = new Lang.Class({
+    Name: 'AlternateTab.WindowList',
+    Extends: AltTab.SwitcherList,
 
     _init : function(windows) {
-        AltTab.SwitcherList.prototype._init.call(this, true);
+        this.parent(true);
 
         let activeWorkspace = global.screen.get_active_workspace();
         this._labels = new Array();
@@ -593,29 +484,47 @@ WindowList.prototype = {
     addSeparator: function () {
         this._separator=null;
     }
+});
+
+const MODES = {
+    all_thumbnails: AltTabPopupAllThumbnails,
+    workspace_icons: AltTabPopupWorkspaceIcons,
 };
 
-function init(metadata) {
-    imports.gettext.bindtextdomain('gnome-shell-extensions', GLib.build_filenamev([metadata.path, 'locale']));
+function doAltTab(display, screen, window, binding) {
+    let behaviour = settings.get_string(SETTINGS_BEHAVIOUR_KEY);
+
+    // alt-tab having no effect is unexpected, even with wrong settings
+    if (!(behaviour in MODES))
+        behaviour = 'all_thumbnails';
+
+    if (Main.wm._workspaceSwitcherPopup)
+        Main.wm._workspaceSwitcherPopup.actor.hide();
+
+    let modifiers = binding.get_modifiers()
+    let backwards = modifiers & Meta.VirtualModifier.SHIFT_MASK;
+
+    let constructor = MODES[behaviour];
+    let popup = new constructor();
+    if (!popup.show(backwards, binding.get_name(), binding.get_mask()))
+        popup.destroy();
 }
 
-function doAltTab(shellwm, binding, mask, window, backwards) {
-    let behaviour = BEHAVIOUR;
-    if(behaviour in MODES) {
-        MODES[behaviour](shellwm, binding, mask, window, backwards);
-    }
+function init(metadata) {
+    Convenience.initTranslations();
+    settings = Convenience.getSettings();
 }
 
 function enable() {
-    Main.wm.setKeybindingHandler('switch_windows', doAltTab);
-    Main.wm.setKeybindingHandler('switch_group', doAltTab);
-    Main.wm.setKeybindingHandler('switch_windows_backward', doAltTab);
-    Main.wm.setKeybindingHandler('switch_group_backward', doAltTab);
+    Meta.keybindings_set_custom_handler('switch-windows', doAltTab);
+    Meta.keybindings_set_custom_handler('switch-group', doAltTab);
+    Meta.keybindings_set_custom_handler('switch-windows-backward', doAltTab);
+    Meta.keybindings_set_custom_handler('switch-group-backward', doAltTab);
 }
 
 function disable() {
-    Main.wm.setKeybindingHandler('switch_windows', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
-    Main.wm.setKeybindingHandler('switch_group', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
-    Main.wm.setKeybindingHandler('switch_windows_backward', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
-    Main.wm.setKeybindingHandler('switch_group_backward', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
+    Meta.keybindings_set_custom_handler('switch-windows', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
+    Meta.keybindings_set_custom_handler('switch-group', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
+    Meta.keybindings_set_custom_handler('switch-windows-backward', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
+    Meta.keybindings_set_custom_handler('switch-group-backward', Lang.bind(Main.wm, Main.wm._startAppSwitcher));
 }
