@@ -11,16 +11,19 @@ const St = imports.gi.St;
 
 const Main = imports.ui.main;
 
-// list of applications and workspace pairs
-// format: "desktop-file-id.desktop:<num>"
-const APPLICATIONS = [ ];
+const ExtensionUtils = imports.misc.extensionUtils;
+const Me = ExtensionUtils.getCurrentExtension();
+const Convenience = Me.imports.convenience;
 
-function WindowMover() {
-    this._init();
-}
+const SETTINGS_KEY = 'application-list';
+
+let settings;
+
+const WindowMover = new Lang.Class({
+    Name: 'AutoMoveWindows.WindowMover',
 
-WindowMover.prototype = {
     _init: function() {
+        this._settings = settings;
         this._windowTracker = Shell.WindowTracker.get_default();
 
         let display = global.screen.get_display();
@@ -46,7 +49,7 @@ WindowMover.prototype = {
         if (!this._windowTracker.is_window_interesting(window))
             return;
 
-        let spaces = APPLICATIONS;
+        let spaces = this._settings.get_strv(SETTINGS_KEY);
 
         let app = this._windowTracker.get_window_app(window);
         if (!app) {
@@ -74,13 +77,14 @@ WindowMover.prototype = {
             }
         }
     }
-}
+});
 
 let prevCheckWorkspaces;
 let winMover;
 
-function init(extensionMeta) {
-    // do nothing here
+function init() {
+    Convenience.initTranslations();
+    settings = Convenience.getSettings();
 }
 
 function enable() {
