@@ -1,5 +1,4 @@
 // Drive menu extension
-const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
 const Lang = imports.lang;
 const St = imports.gi.St;
@@ -13,15 +12,16 @@ const Panel = imports.ui.panel;
 const PanelMenu = imports.ui.panelMenu;
 const PopupMenu = imports.ui.popupMenu;
 
-function DriveMenuItem(place) {
-    this._init(place);
-}
+const ExtensionUtils = imports.misc.extensionUtils;
+const Me = ExtensionUtils.getCurrentExtension();
+const Convenience = Me.imports.convenience;
 
-DriveMenuItem.prototype = {
-    __proto__: PopupMenu.PopupBaseMenuItem.prototype,
+const DriveMenuItem = new Lang.Class({
+    Name: 'DriveMenu.DriveMenuItem',
+    Extends: PopupMenu.PopupBaseMenuItem,
 
     _init: function(place) {
-	PopupMenu.PopupBaseMenuItem.prototype._init.call(this);
+	this.parent();
 
 	this.place = place;
 
@@ -43,23 +43,19 @@ DriveMenuItem.prototype = {
     activate: function(event) {
 	this.place.launch({ timestamp: event.get_time() });
 
-	PopupMenu.PopupBaseMenuItem.prototype.activate.call(this, event);
+	this.parent(event);
     }
-};
+});
 
-function DriveMenu() {
-    this._init();
-}
-
-DriveMenu.prototype = {
-    __proto__: PanelMenu.SystemStatusButton.prototype,
+const DriveMenu = new Lang.Class({
+    Name: 'DriveMenu.DriveMenu',
+    Extends: PanelMenu.SystemStatusButton,
 
     _init: function() {
-	// is 'media-eject' better?
-	PanelMenu.SystemStatusButton.prototype._init.call(this, 'media-optical');
+	this.parent('media-eject');
 
 	this._manager = Main.placesManager;
-	this._manager.connect('mounts-updated', Lang.bind(this, this._update));
+	this._updatedId = this._manager.connect('mounts-updated', Lang.bind(this, this._update));
 
 	this._contentSection = new PopupMenu.PopupMenuSection();
 	this.menu.addMenuItem(this._contentSection);
@@ -88,11 +84,16 @@ DriveMenu.prototype = {
 
 	this.actor.visible = any;
     },
-}
 
-// Put your extension initialization code here
-function init(metadata) {
-    imports.gettext.bindtextdomain('gnome-shell-extensions', GLib.build_filenamev([metadata.path, 'locale']));
+    destroy: function() {
+	this._manager.disconnect(this._updatedId);
+
+	this.parent();
+    },
+});
+
+function init() {
+    Convenience.initTranslations();
 }
 
 let _indicator;
