@@ -20,16 +20,9 @@ const Tweener = imports.ui.tweener;
 const Workspace = imports.ui.workspace;
 const WindowPositionFlags = Workspace.WindowPositionFlags;
 
-const WindowPlacementStrategy = {
-    NATURAL: 0,
-    GRID: 1,
-};
-
-/* Begin user settings */
-const PLACEMENT_STRATEGY = WindowPlacementStrategy.NATURAL;
-const USE_MORE_SCREEN = true;
-const WINDOW_CAPTIONS_ON_TOP = true;
-/* End user settings - do not change anything below this line */
+const ExtensionUtils = imports.misc.extensionUtils;
+const Me = ExtensionUtils.getCurrentExtension();
+const Convenience = Me.imports.convenience;
 
 // testing settings for natural window placement strategy:
 const WINDOW_PLACEMENT_NATURAL_FILLGAPS = true;                     // enlarge windows at the end to fill gaps         // not implemented yet
@@ -40,6 +33,9 @@ const WINDOW_PLACEMENT_NATURAL_MAX_TRANSLATIONS = 5000;             // safety li
 
 const PLACE_WINDOW_CAPTIONS_ON_TOP = true;                          // place window titles in overview on top of windows with overlap parameter
 
+const BUTTON_LAYOUT_SCHEMA = 'org.gnome.shell.overrides';
+const BUTTON_LAYOUT_KEY = 'button-layout';
+
 function injectToFunction(parent, name, func) {
     let origin = parent[name];
     parent[name] = function() {
@@ -52,15 +48,16 @@ function injectToFunction(parent, name, func) {
 }
 const WORKSPACE_BORDER_GAP = 10;                                    // gap between the workspace area and the workspace selector
 
-function Rect(x, y, width, height) {
-    [this.x, this.y, this.width, this.height] = arguments;
-}
+const Rect = new Lang.Class({
+    Name: 'NativeWindowPlacement.Rect',
+
+    _init: function(x, y, width, height) {
+        [this.x, this.y, this.width, this.height] = [x, y, width, height];
+    },
 
-Rect.prototype = {
     /**
      * used in _calculateWindowTransformationsNatural to replace Meta.Rectangle that is too slow.
      */
-
     copy: function() {
         return new Rect(this.x, this.y, this.width, this.height);
     },
@@ -109,22 +106,25 @@ Rect.prototype = {
         this.x += dx;
         this.y += dy;
     }
-};
+});
 
 let winInjections, workspaceInjections, connectedSignals;
 
 function resetState() {
     winInjections = { };
     workspaceInjections = { };
-    workViewInjections = { };
     connectedSignals = [ ];
 }
 
 function enable() {
     resetState();
 
-    let placementStrategy = PLACEMENT_STRATEGY;
-    let useMoreScreen = USE_MORE_SCREEN;
+    let settings = Convenience.getSettings();
+    let useMoreScreen = settings.get_boolean('use-more-screen');
+    signalId = settings.connect('changed::use-more-screen', function() {
+        useMoreScreen = settings.get_boolean('use-more-screen');
+    });
+    connectedSignals.push({ obj: settings, id: signalId });
 
     /**
      * _calculateWindowTransformationsNatural:
@@ -297,25 +297,6 @@ function enable() {
     }
     workspaceInjections['_calculateWindowTransformationsNatural'] = undefined;
 
-    /**
-     * _calculateWindowTransformationsGrid:
-     * @clones: Array of #MetaWindow
-     *
-     * Returns clones with matching target coordinates and scales to arrange windows in a grid.
-     */
-    Workspace.Workspace.prototype._calculateWindowTransformationsGrid = function(clones) {
-        let slots = this._computeAllWindowSlots(clones.length);
-        clones = this._orderWindowsByMotionAndStartup(clones, slots);
-        let targets = [];
-
-        for (let i = 0; i < clones.length; i++) {
-            targets[i] = this._computeWindowLayout(clones[i].metaWindow, slots[i]);
-        }
-
-        return [clones, targets];
-    }
-    workspaceInjections['_calculateWindowTransformationsGrid'] = undefined;
-
     /**
      * positionWindows:
      * @flags:
@@ -340,17 +321,7 @@ function enable() {
 	let targets = [];
         let scales = [];
 
-        switch (placementStrategy) {
-        case WindowPlacementStrategy.NATURAL:
-            [clones, targets] = this._calculateWindowTransformationsNatural(clones);
-            break;
-        default:
-            log ('Invalid window placement strategy');
-            placementStrategy = WindowPlacementStrategy.GRID;
-        case WindowPlacementStrategy.GRID:
-            [clones, targets] = this._calculateWindowTransformationsGrid(clones);
-            break;
-        }
+        [clones, targets] = this._calculateWindowTransformationsNatural(clones);
 
 	let currentWorkspace = global.screen.get_active_workspace();
         let isOnCurrentWorkspace = this.metaWorkspace == null || this.metaWorkspace == currentWorkspace;
@@ -367,7 +338,7 @@ function enable() {
             if (clone.inDrag)
                 continue;
 
-            if (overlay)
+            if (overlay && initialPositioning)
                 overlay.hide();
             if (animate && isOnCurrentWorkspace) {
                 if (!metaWindow.showing_on_its_workspace()) {
@@ -390,27 +361,18 @@ function enable() {
                                      });
                 }
 
-                Tweener.addTween(clone.actor,
-                                 { x: x,
-                                   y: y,
-                                   scale_x: scale,
-                                   scale_y: scale,
-                                   time: Overview.ANIMATION_TIME,
-                                   transition: 'easeOutQuad',
-                                   onComplete: Lang.bind(this, function() {
-				       this._showWindowOverlay(clone, overlay, true);
-                                   })
-                                 });
+                this._animateClone(clone, overlay, x, y, scale, initialPositioning);
             } else {
                 clone.actor.set_position(x, y);
                 clone.actor.set_scale(scale, scale);
+                this._updateWindowOverlayPositions(clone, overlay, x, y, scale, false);
                 this._showWindowOverlay(clone, overlay, isOnCurrentWorkspace);
             }
         }
     }
 
     /// position window titles on top of windows in overlay ////
-    if (WINDOW_CAPTIONS_ON_TOP)  {
+    if (settings.get_boolean('window-captions-on-top'))  {
         winInjections['_init'] = Workspace.WindowOverlay.prototype._init;
 	Workspace.WindowOverlay.prototype._init = function(windowClone, parentActor) {
             let metaWindow = windowClone.metaWindow;
@@ -466,26 +428,46 @@ function enable() {
 	},
 
         winInjections['updatePositions'] = Workspace.WindowOverlay.prototype.updatePositions;
-	Workspace.WindowOverlay.prototype.updatePositions = function(cloneX, cloneY, cloneWidth, cloneHeight) {
+	Workspace.WindowOverlay.prototype.updatePositions = function(cloneX, cloneY, cloneWidth, cloneHeight, animate) {
             let button = this.closeButton;
             let title = this.title;
 
+            let settings = new Gio.Settings({ schema: BUTTON_LAYOUT_SCHEMA });
+            let layout = settings.get_string(BUTTON_LAYOUT_KEY);
+            let rtl = Clutter.get_default_text_direction() == Clutter.TextDirection.RTL;
+
+            let split = layout.split(":");
+            let side;
+            if (split[0].indexOf("close") > -1)
+                side = rtl ? St.Side.RIGHT : St.Side.LEFT;
+            else
+                side = rtl ? St.Side.LEFT : St.Side.RIGHT;
+
             let buttonX;
             let buttonY = cloneY - (button.height - button._overlap);
-            if (St.Widget.get_default_direction() == St.TextDirection.RTL)
-		buttonX = cloneX - (button.width - button._overlap);
+            if (side == St.Side.LEFT)
+                buttonX = cloneX - (button.width - button._overlap);
             else
-		buttonX = cloneX + (cloneWidth - button._overlap);
+                buttonX = cloneX + (cloneWidth - button._overlap);
 
-            button.set_position(Math.floor(buttonX), Math.floor(buttonY));
+            if (animate)
+                this._animateOverlayActor(button, Math.floor(buttonX), Math.floor(buttonY), button.width);
+            else
+                button.set_position(Math.floor(buttonX), Math.floor(buttonY));
 
             if (!title.fullWidth)
-		title.fullWidth = title.width;
-            title.width = Math.min(title.fullWidth, cloneWidth);
+                title.fullWidth = title.width;
+            let titleWidth = Math.min(title.fullWidth, cloneWidth);
 
-            let titleX = cloneX + (cloneWidth - title.width) / 2;
-	    let titleY = cloneY - title.height + title._overlap;
-            title.set_position(Math.floor(titleX), Math.floor(titleY));
+            let titleX = cloneX + (cloneWidth - titleWidth) / 2;
+            let titleY = cloneY - title.height + title._overlap;
+
+            if (animate)
+                this._animateOverlayActor(title, Math.floor(titleX), Math.floor(titleY), titleWidth);
+            else {
+                title.width = titleWidth;
+                title.set_position(Math.floor(titleX), Math.floor(titleY));
+            }
 	},
 
         winInjections['_onStyleChanged'] = Workspace.WindowOverlay.prototype._onStyleChanged;
