@@ -14,17 +14,18 @@ const Panel = imports.ui.panel;
 const Gettext = imports.gettext.domain('gnome-shell-extensions');
 const _ = Gettext.gettext;
 
+const ExtensionUtils = imports.misc.extensionUtils;
+const Me = ExtensionUtils.getCurrentExtension();
+const Convenience = Me.imports.convenience;
+
 const PLACE_ICON_SIZE = 22;
 
-function PlacesMenu() {
-    this._init.apply(this, arguments);
-}
-
-PlacesMenu.prototype = {
-    __proto__: PanelMenu.SystemStatusButton.prototype,
+const PlacesMenu = new Lang.Class({
+    Name: 'PlacesMenu.PlacesMenu',
+    Extends: PanelMenu.SystemStatusButton,
 
     _init: function() {
-        PanelMenu.SystemStatusButton.prototype._init.call(this, 'folder');
+        this.parent('folder');
 
         this.defaultItems = [];
         this.bookmarkItems = [];
@@ -33,11 +34,19 @@ PlacesMenu.prototype = {
         this._bookmarksSection = new PopupMenu.PopupMenuSection();
         this.menu.addMenuItem(this._bookmarksSection);
         this._createBookmarks();
-        this._devicesMenuItem = new PopupMenu.PopupSubMenuMenuItem('Removable Devices');
+        this._devicesMenuItem = new PopupMenu.PopupSubMenuMenuItem(_("Removable Devices"));
         this.menu.addMenuItem(this._devicesMenuItem);
         this._createDevices();
-        Main.placesManager.connect('bookmarks-updated',Lang.bind(this,this._redisplayBookmarks));
-        Main.placesManager.connect('mounts-updated',Lang.bind(this,this._redisplayDevices));
+
+        this._bookmarksId = Main.placesManager.connect('bookmarks-updated',Lang.bind(this,this._redisplayBookmarks));
+        this._mountsId = Main.placesManager.connect('mounts-updated',Lang.bind(this,this._redisplayDevices));
+    },
+
+    destroy: function() {
+        Main.placesManager.disconnect(this._bookmarksId);
+        Main.placesManager.disconnect(this._mountsId);
+
+        this.parent();
     },
 
     _redisplayBookmarks: function(){
@@ -108,13 +117,12 @@ PlacesMenu.prototype = {
 
     _clearDevices : function(){
         this._devicesMenuItem.menu.removeAll();
-        this.DeviceItems = [];
+        this.deviceItems = [];
     },
-};
+});
 
-
-function init(metadata) {
-    imports.gettext.bindtextdomain('gnome-shell-extensions', GLib.build_filenamev([metadata.path, 'locale']));
+function init() {
+    Convenience.initTranslations();
 }
 
 let _indicator;
