@@ -139,6 +139,9 @@ function enable() {
     workViewInjections['_onKeyRelease'] = undefined;
 
     WorkspacesView.WorkspacesView.prototype._onKeyPress = function(s, o) {
+        if(Main.overview._viewSelector._activeTab.id != 'windows')
+            return false;
+
         if ((o.get_key_symbol() == Clutter.KEY_Alt_L ||
              o.get_key_symbol() == Clutter.KEY_Alt_R)
             && !this._pickWorkspace) {
@@ -175,10 +178,14 @@ function enable() {
                 return false;
             }
 
-            let c = o.get_key_symbol() - Clutter.KEY_0;
+            let c = o.get_key_symbol() - Clutter.KEY_KP_0;
             if (c > 9 || c <= 0) {
-                this._hideTooltips();
-                return false;
+                c = o.get_key_symbol() - Clutter.KEY_0;
+                if (c > 9 || c <= 0) {
+                    this._hideTooltips();
+                    global.log(c);
+                    return false;
+                }
             }
 
             let win = this._workspaces[this._active].getWindowWithTooltip(c);
@@ -190,10 +197,13 @@ function enable() {
             return true;
         }
         if (this._pickWorkspace) {
-            let c = o.get_key_symbol() - Clutter.KEY_0;
+            let c = o.get_key_symbol() - Clutter.KEY_KP_0;
             if (c > 9 || c <= 0) {
-                this._hideWorkspacesTooltips();
-                return false;
+                c = o.get_key_symbol() - Clutter.KEY_0;
+                if (c > 9 || c <= 0) {
+                    this._hideWorkspacesTooltips();
+                    return false;
+                }
             }
 
             let workspace = this._workspaces[c - 1];
@@ -217,7 +227,7 @@ function enable() {
     winInjections['updatePositions'] = injectToFunction(Workspace.WindowOverlay.prototype, 'updatePositions', function(cloneX, cloneY, cloneWidth, cloneHeight) {
         let textX = cloneX - 2;
         let textY = cloneY - 2;
-        this._text.set_position(Math.floor(textX), Math.floor(textY));
+        this._text.set_position(Math.floor(textX) + 5, Math.floor(textY) + 5);
         this._text.raise_top();
     });
 
